package betterwithmods.common.blocks;

import betterwithmods.api.block.IMultiVariants;
import betterwithmods.client.BWCreativeTabs;
import betterwithmods.common.BWMBlocks;
import betterwithmods.common.BWMItems;
import betterwithmods.common.items.ItemMaterial;
import betterwithmods.module.hardcore.world.HCStumping;
import com.google.common.collect.Lists;
import net.minecraft.block.Block;
import net.minecraft.block.BlockLog;
import net.minecraft.block.BlockPlanks;
import net.minecraft.block.SoundType;
import net.minecraft.block.material.MapColor;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.init.Blocks;
import net.minecraft.item.ItemStack;
import net.minecraft.util.NonNullList;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.Explosion;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;

import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.List;

import static net.minecraft.block.BlockPlanks.VARIANT;

/**
 * @author Koward
 */
public class BlockStump extends Block implements IMultiVariants {
    private static final int MIN_TRUNK_HEIGHT = 2;

    public BlockStump() {
        super(Material.field_151575_d);
        this.func_149647_a(BWCreativeTabs.BWTAB);
        this.func_149711_c(40F);
        this.func_149752_b(10f);
        this.func_149672_a(SoundType.field_185848_a);
        this.func_180632_j(this.field_176227_L.func_177621_b().func_177226_a(field_176383_a, BlockPlanks.EnumType.OAK));
        this.setHarvestLevel("axe", 0);
        Blocks.field_150480_ab.func_180686_a(this, 5, 5);
    }

    /**
     * Whether a stump could be placed or not.
     *
     * @param worldIn  The world where stump will be placed.
     * @param position The position at the base of the tree.
     * @return true when the trunk is high enough.
     */
    public static boolean canPlaceStump(World worldIn, BlockPos position) {
        for (int i = 0; i < MIN_TRUNK_HEIGHT; i++) {
            IBlockState state = worldIn.func_180495_p(position.func_177981_b(i));
            if (!(state.func_177230_c() == Blocks.field_150364_r ||
                    state.func_177230_c() == Blocks.field_150363_s)) {
                return false;
            }
            if (i == 0 && state.func_177228_b().containsKey(BlockLog.field_176299_a)
                    && state.func_177229_b(BlockLog.field_176299_a) != BlockLog.EnumAxis.Y) {
                return false;
            }
        }
        return true;
    }

    /**
     * Return a stump states that matches the log given in parameter.
     *
     * @param log The log of the tree.
     * @return If possible a state with the same {@link BlockPlanks.EnumType} as the log, null otherwise.
     */
    @Nullable
    public static IBlockState getStump(IBlockState log) {
        BlockPlanks.EnumType variant = HCStumping.getWoodType(log);
        if (variant == null) return null;
        return BWMBlocks.STUMP.func_176223_P().func_177226_a(BlockPlanks.field_176383_a, variant);
    }

    @Override
    public boolean isWood(IBlockAccess world, BlockPos pos) {
        return true;
    }

    @Override
    protected ItemStack func_180643_i(IBlockState state) {
        if (state.func_177228_b().containsKey(BlockPlanks.field_176383_a)) {
            BlockPlanks.EnumType type = state.func_177229_b(BlockPlanks.field_176383_a);
            if (type.func_176839_a() < 4)
                return new ItemStack(Blocks.field_150364_r, 1, type.func_176839_a());
            else
                return new ItemStack(Blocks.field_150363_s, 1, type.func_176839_a() - 4);
        }
        return ItemStack.field_190927_a;
    }

    @Override
    public List<ItemStack> getDrops(IBlockAccess world, BlockPos pos, IBlockState state, int fortune) {
        List<ItemStack> ret = Lists.newArrayList();

        if (state.func_177228_b().containsKey(BlockPlanks.field_176383_a)) {
            ret.add(new ItemStack(BWMItems.BARK, 1, state.func_177229_b(BlockPlanks.field_176383_a).func_176839_a()));
        }
        ret.add(ItemMaterial.getMaterial(ItemMaterial.EnumMaterial.SAWDUST, 16));
        return ret;
    }

    @Override
    public boolean func_149659_a(Explosion explosionIn) {
        return false;
    }

    @Override
    public void func_180652_a(World worldIn, BlockPos pos, Explosion explosionIn) {
        for (int i = 0; i < 4; ++i) {
            func_180635_a(worldIn, pos, ItemMaterial.getMaterial(ItemMaterial.EnumMaterial.SAWDUST, 1));
        }
    }

    @Override
    public MapColor func_180659_g(IBlockState state, IBlockAccess worldIn, BlockPos pos) {
        BlockPlanks.EnumType type = state.func_177229_b(field_176383_a);
        return type.func_181070_c();
    }

    @Override
    public void func_149666_a(CreativeTabs itemIn, NonNullList<ItemStack> items) {
        for (BlockPlanks.EnumType type : BlockPlanks.EnumType.values()) {
            items.add(new ItemStack(this, 1, type.func_176839_a()));
        }
    }

    @Override
    public IBlockState func_176203_a(int meta) {
        return this.func_176223_P().func_177226_a(field_176383_a, BlockPlanks.EnumType.func_176837_a(meta));
    }

    @Override
    public int func_176201_c(IBlockState state) {
        return state.func_177229_b(field_176383_a).func_176839_a();
    }

    @Override
    protected BlockStateContainer func_180661_e() {
        return new BlockStateContainer(this, field_176383_a);
    }

    @Override
    public int func_180651_a(IBlockState state) {
        return state.func_177229_b(field_176383_a).func_176839_a();
    }

    @Override
    public String[] getVariants() {
        ArrayList<String> variants = new ArrayList<>();
        for (BlockPlanks.EnumType blockplanks$enumtype : BlockPlanks.EnumType.values()) {
            variants.add("variant=" + blockplanks$enumtype.func_176610_l());
        }
        return variants.toArray(new String[BlockPlanks.EnumType.values().length]);
    }


    @Override
    public boolean isToolEffective(String type, IBlockState state) {
        return false;
    }
}
